from flax import linen
import tensorial.gcnn
from tensorial.gcnn import atomic
import tensorial.nn

from . import mlip
from .mlip import keys


def equivariant_mlip(
    r_max: float,
    atomic_numbers: list[int],
    nn: linen.Module,
    ell_max: int = 1,
    forces: bool = False,
    stresses: bool = False,
):
    current = nn
    layers = [
        atomic.SpeciesTransform(atomic_numbers),
        tensorial.gcnn.EdgeVectors(),
    ]

    if stresses:
        layers.extend(
            [
                tensorial.gcnn.Grad(
                    nn,
                    of=f"globals.{keys.PREDICTED_ENERGY}",
                    wrt="edges.edge_vectors",
                    out_field=f"edges.{keys.PREDICTED_FORCES}",
                    sign=-1,
                ),
                mlip.AtomicVirialsStresses(),
            ]
        )

    if forces:
        nn = tensorial.gcnn.Grad(
            nn,
            of=f"globals.{keys.PREDICTED_ENERGY}",
            wrt=f"nodes.{tensorial.gcnn.POSITIONS}",
            out_field=f"nodes.{keys.PREDICTED_FORCES}",
            sign=-1,
        )

    return tensorial.nn.Sequential(layers)
